/*
=============
Author: Lucas Josino
Github: https://github.com/LucasPJS
Website: https://lucasjosino.com/
=============
Plugin/Id: on_audio_room#4
Homepage: https://github.com/LucasPJS/on_audio_room
Pub: https://pub.dev/packages/on_audio_room
License: https://github.com/LucasPJS/on_audio_room/blob/main/LICENSE
Copyright: © 2021, Lucas Josino. All rights reserved.
=============
*/

package com.lucasjosino.on_audio_room

import android.annotation.SuppressLint
import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.lucasjosino.on_audio_room.databases.OnAudioDatabase
import com.lucasjosino.on_audio_room.sources.global.OnGlobalSource
import com.lucasjosino.on_audio_room.sources.playlist.OnPlaylistSource
import com.lucasjosino.on_audio_room.sources.query.OnQuerySource
import com.lucasjosino.on_audio_room.utils.*

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/** OnAudioRoomPlugin */
class OnAudioRoomPlugin : FlutterPlugin, MethodCallHandler, ViewModel() {

    //Dart <-> Kotlin communication
    private val channelName = "com.lucasjosino.on_audio_room"
    private lateinit var channel: MethodChannel

    //Main parameters
    @SuppressLint("StaticFieldLeak")
    private lateinit var pContext: Context

    //Database parameters
    private val defaultDatabaseName = "on_audio_db"
    private val defaultDatabaseLimit: Int = 50
    private val onSharedPrefKeyDbName = "on_audio_db_name"
    private val onSharedPrefKeyDbLimit = "on_audio_db_limit"

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        this.pContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, channelName)
        channel.setMethodCallHandler(this)
    }

    //Clear all database information.
    private fun clearAllDatabase(context: Context): Boolean {
        return try {
            viewModelScope.launch(Dispatchers.IO) {
                OnAudioDatabase.initDatabase(context).clearAllTables()
            }
            true
        } catch (e: Exception) {
            Log.i(channelError, e.toString())
            false
        }
    }

    //Get databaseName defined using "setDatabaseName" or [defaultDatabaseName].
    fun getDbName(context: Context): String? = context.getSharedPreferences(
        "on_audio_room",
        Context.MODE_PRIVATE
    ).getString(onSharedPrefKeyDbName, defaultDatabaseName)

    //Get databaseLimit defined using "setDatabaseLimit" or [defaultDatabaseLimit].
    fun getDbLimit(context: Context): Int = context.getSharedPreferences(
        "on_audio_room",
        Context.MODE_PRIVATE
    ).getInt(onSharedPrefKeyDbLimit, defaultDatabaseLimit)

    //Main method to start communication.
    //
    //- All methods used to Add/Delete/Query has your own "Room". And this will redirect them.
    //- Methods about database information/limit will be executed here or in [OnDatabaseUtils].
    //
    //Count:
    //
    //- 28 [calls] in total.
    //- 46 [methods] in total.
    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            //Database methods
            "getDatabaseName" -> result.success(getDbName(pContext) ?: defaultDatabaseName)
            "getDatabaseLimit" -> result.success(getDbLimit(pContext))
            "getDatabaseStatus" -> result.success(OnAudioDatabase.initDatabase(pContext).isOpen)
            "setDatabaseName" -> setDatabaseName(pContext, result, call)
            "setDatabaseLimit" -> setDatabaseLimit(pContext, result, call)
            "resetDatabaseName" -> resetDatabaseName(pContext, result)
            "resetDatabaseLimit" -> resetDatabaseLimit(pContext, result)
            "closeDatabase" -> result.success(OnAudioDatabase.closeDatabase(pContext))
            "resetAllDatabase" -> result.success(clearAllDatabase(pContext))
//            "deleteDatabase" -> result.success(deleteDatabaseFile(pContext))

            //Query methods
            //[0] -> queryFrom methods ; [1] -> queryAllFrom methods
            "queryFromFavorites",
            "queryFromLastPlayed",
            "queryFromMostPlayed",
            "queryFromPlaylist" -> OnQuerySource(pContext).initQuerySource(result, call, 0)

            //QueryAll methods
            "queryAllFromFavorites",
            "queryAllFromLastPlayed",
            "queryAllFromMostPlayed",
            "queryAllFromPlaylist" -> OnQuerySource(pContext).initQuerySource(result, call, 1)

            //Playlist methods
            "createPlaylist",
            "deletePlaylist",
            "renamePlaylist",
            "queryPlaylist",
            "queryAllPlaylists" -> OnPlaylistSource(pContext).initPlaylistSource(result, call)

            //Global methods(AddTo, AddAllTo, DeleteFrom, DeleteAllFrom and CheckIn)
            else -> OnGlobalSource(pContext).initGlobalSource(result, call)
        }
    }

    //This is only important for Dart <-> Kotlin communication
    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
