package com.lucasjosino.on_audio_room.sources.global

import android.annotation.SuppressLint
import android.content.Context
import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.lucasjosino.on_audio_room.databases.OnAudioDatabase
import com.lucasjosino.on_audio_room.utils.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@SuppressLint("StaticFieldLeak")
class OnGlobalSource(private val context: Context) : ViewModel() {

    //Main parameters.
    private var type: Int = -1
    private var songId: Number? = null
    private var playlistId: Number? = null
    private var entity: MutableMap<String, Any>? = HashMap()
    private var listOfEntity: ArrayList<MutableMap<String, Any>>? = ArrayList()

    //Get database configuration.
    private fun initDb(): OnAudioDatabase = OnAudioDatabase.initDatabase(context)

    //This function is the main and initial to all calls that have almost the same process.
    fun initGlobalSource(result: MethodChannel.Result, call: MethodCall) {
        //Method [type] *never* should be null.
        this.type = call.argument<Int>("type")
            ?: throw Exception("[$channelError]: [type] from [initGlobalSource] have a null value.")

        //All others methods can be null.
        //Every [method] have your specific require variable, and cannot be null when calling the method.
        //
        //- [addTo] -> entity and playlistId(*)
        //- [addAllTo] -> listOfEntity and playlistId(*)
        //- [deleteFrom] -> songId and playlistId(*)
        //- [deleteAllFrom] -> playlistId(*)
        //- [checkIn] -> songId and playlistId(*)
        //- [queryAllIdsFrom] -> playlistId(*)
        //
        // (*) -> Only when calling [RoomType.PLAYLIST_SONGS](type == 3))
        this.songId = call.argument<Number>("songId")
        this.playlistId = call.argument<Number>("playlistId")
        this.entity = call.argument("entity")
        this.listOfEntity = call.argument("listOfEntity")

        //Query everything in the Background it's necessary for better performance
        viewModelScope.launch {
            //Start loading
            val dbResult: Any? = when (call.method) {
                "addTo" -> addTo()
                "addAllTo" -> addAllTo()
                "deleteFrom" -> deleteFrom()
                "deleteAllFrom" -> deleteAllFrom()
                "checkIn" -> checkIn()
                "queryAllIdsFrom" -> queryAllIdsFrom()
                else -> throw Exception("[$channelError]: [initGlobalSource] have a null value.")
            }

            //Flutter UI will start, but, information still loading.
            //
            //We check [dbResult] to make sure that return will *never* be null.
            //If [dbResult] return null, something is missing and plugin will throw a error.
            if (!dbResult.isNull()) result.success(dbResult) else
                result.error(
                    channelError,
                    "[${call.method}] was called but not all parameters was defined.",
                    "Define all required parameters."
                )
        }
    }

    /*
     * This method is used to add a specific song(entity) to a specific database.
     *
     * @param entity Variable that contains all information about song.
     * @param playlistId Variable that define which playlist this entity will be added.
     * @return [Null] if some necessary variable is null or a [Long] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun addTo(): Long? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (entity.isNullOrEmpty() || type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(addToFavorites)
        //- [1] -> LastPlayed(addToLastPlayed)
        //- [2] -> MostPlayed(addToMostPlayed)
        //- [3] -> Playlist songs(addToPlaylist)
        //
        return@withContext when (type) {
            0 -> initDb().onFavoritesDao().addToFavorites(entity!!.toFavoritesEntity())
            1 -> initDb().onLastPlayedDao().addToLastPlayed(entity!!.toLastPlayedEntity())
            2 -> initDb().onMostPlayedDao().addToMostPlayed(entity!!.toMostPlayedEntity())
            3 -> initDb().onPlaylistSongDao().addToPlaylist(entity!!.toPlaylistSongsEntity())
            //[addTo] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [addTo] have a null value.")
        }
    }

    /*
     * This method is used to add multiples songs(entities) to a specific database.
     *
     * @param listOfEntity Variable that contains all information about all song.
     * @param playlistId Variable that define which playlist this entity will be added.
     * @return [Null] if some necessary variable is null or a list of [Long] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun addAllTo(): ArrayList<Long>? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (listOfEntity.isNullOrEmpty() ||
            type == 3 && playlistId.isNull()
        ) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(addToFavorites)
        //- [1] -> LastPlayed(addToLastPlayed)
        //- [2] -> MostPlayed(addToMostPlayed)
        //- [3] -> Playlist songs(addToPlaylist)
        //
        val tempList: ArrayList<Long> = ArrayList()
        when (type) {
            //Every [type] will pass to the same process:
            //
            //- 1º -> For each item in [listOfEntity] we will convert to [type]Entity.
            //- 2º -> Add to the database.
            //- 3º -> This method will return a Long(id) for every "add", we get this Long and
            //        add to a temporary list[tempList].
            //- 4º -> After this, return to [dbResult] with this [tempList].
            0 -> listOfEntity!!.forEach {
                tempList.add(initDb().onFavoritesDao().addToFavorites(it.toFavoritesEntity()))
            }
            //
            1 -> listOfEntity!!.forEach {
                tempList.add(
                    initDb().onLastPlayedDao().addToLastPlayed(it.toLastPlayedEntity())
                )
            }
            //
            2 -> listOfEntity!!.forEach {
                tempList.add(
                    initDb().onMostPlayedDao().addToMostPlayed(it.toMostPlayedEntity())
                )
            }
            //
            3 -> listOfEntity!!.forEach {
                tempList.add(
                    initDb().onPlaylistSongDao().addToPlaylist(it.toPlaylistSongsEntity())
                )
            }
            //[addAllTo] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [addAllTo] have a null value.")
        }
        return@withContext tempList
    }

    /*
     * This method is used to delete a specific song(entity) from a specific database.
     *
     * @param songId Variable that contains the id of a song.
     * @param playlistId Variable that define which playlist this id will be deleted.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun deleteFrom(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (songId.isNull() || type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(deleteFromFavorites)
        //- [1] -> LastPlayed(deleteFromLastPlayed)
        //- [2] -> MostPlayed(deleteFromMostPlayed)
        //- [3] -> Playlist songs(deleteFromPlaylist)
        //
        return@withContext when (type) {
            0 -> try {
                initDb().onFavoritesDao().deleteFromFavorites(songId!!.toLong()); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            1 -> try {
                initDb().onLastPlayedDao().deleteFromLastPlayed(songId!!.toLong()); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            2 -> try {
                initDb().onMostPlayedDao().deleteFromMostPlayed(songId!!.toLong()); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            3 -> try {
                initDb().onPlaylistSongDao()
                    .deleteFromPlaylist(playlistId!!.toLong(), songId!!.toLong()); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //[deleteFrom] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [deleteFrom] have a null value.")
        }
    }

    /*
     * This method is used to delete [All] songs(entities) from a specific database.
     *
     * @param playlistId Variable that define which playlist this entity will be added.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun deleteAllFrom(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(deleteAllFromFavorites)
        //- [1] -> LastPlayed(deleteAllFromLastPlayed)
        //- [2] -> MostPlayed(deleteAllFromMostPlayed)
        //- [3] -> Playlist songs(deleteAllInPlaylist)
        //
        return@withContext when (type) {
            0 -> try {
                initDb().onFavoritesDao().deleteAllFromFavorites(); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            1 -> try {
                initDb().onLastPlayedDao().deleteAllFromLastPlayed(); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            2 -> try {
                initDb().onMostPlayedDao().deleteAllFromMostPlayed(); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            3 -> try {
                initDb().onPlaylistSongDao().deleteAllInPlaylist(playlistId!!.toLong()); true
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //[deleteAllFrom] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [deleteAllFrom] have a null value.")
        }
    }

    /*
     * This method is used to check a specific song(entity) from a specific database.
     *
     * @param songId Variable that contains the id of a song.
     * @param playlistId Variable that define which playlist this id will be deleted.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun checkIn(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (songId.isNull() || type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(checkInFavorites)
        //- [1] -> LastPlayed(checkInLastPlayed)
        //- [2] -> MostPlayed(checkInMostPlayed)
        //- [3] -> Playlist songs(checkInPlaylist)
        //
        return@withContext when (type) {
            0 -> try {
                initDb().onFavoritesDao().checkInFavorites(songId!!.toLong())
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            1 -> try {
                initDb().onLastPlayedDao().checkInLastPlayed(songId!!.toLong())
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            2 -> try {
                initDb().onMostPlayedDao().checkInMostPlayed(songId!!.toLong())
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //
            3 -> try {
                initDb().onPlaylistSongDao()
                    .checkInPlaylist(playlistId!!.toLong(), songId!!.toLong())
            } catch (e: Exception) {
                Log.i(channelError, e.toString()); false
            }
            //[checkIn] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [checkIn] have a null value.")
        }
    }

    /*
     * This method is used to query [All] songs(ids) from a specific database.
     *
     * @param playlistId Variable that define which playlist this id will be deleted.
     * @return [Null] if some necessary variable is null or a list of [Long] if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun queryAllIdsFrom(): ArrayList<Long>? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(queryAllIdsFromFavorites)
        //- [1] -> LastPlayed(queryAllIdsFromLastPlayed)
        //- [2] -> MostPlayed(queryAllIdsFromMostPlayed)
        //- [3] -> Playlist songs(queryAllIdsFromPlaylistSongs)
        //
        return@withContext when (type) {
            0 -> ArrayList(initDb().onFavoritesDao().queryAllIdsFromFavorites())
            1 -> ArrayList(initDb().onLastPlayedDao().queryAllIdsFromLastPlayed())
            2 -> ArrayList(initDb().onMostPlayedDao().queryAllIdsFromMostPlayed())
            3 -> ArrayList(
                initDb().onPlaylistSongDao().queryAllIdsFromPlaylistSongs(playlistId!!.toLong())
            )
            //[queryAllIdsFrom] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [queryAllIdsFrom] have a null value.")
        }
    }
}