package com.lucasjosino.on_audio_room.sources.playlist

import android.annotation.SuppressLint
import android.content.Context
import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.lucasjosino.on_audio_room.OnAudioRoomPlugin
import com.lucasjosino.on_audio_room.databases.OnAudioDatabase
import com.lucasjosino.on_audio_room.databases.playlists.OnPlaylistDao
import com.lucasjosino.on_audio_room.utils.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@SuppressLint("StaticFieldLeak")
class OnPlaylistSource(private val context: Context) : ViewModel() {

    //Main parameters.
    private var playlistId: Number? = null
    private var entity: MutableMap<String, Any>? = HashMap()
    private var playlistNewName: String? = null

    //Get database configuration.
    private fun initPlaylistDb(): OnPlaylistDao =
        OnAudioDatabase.initDatabase(context).onPlaylistDao()

    //Get database query limit.
    private fun getDbLimit(): Int = OnAudioRoomPlugin().getDbLimit(context)

    //This function is the main and initial to all calls based on [playlist].
    fun initPlaylistSource(result: MethodChannel.Result, call: MethodCall) {
        //All others methods can be null.
        //Every [method] have your specific require variable, and cannot be null when
        //calling the method.
        //
        //- [createPlaylist] -> entity
        //- [deletePlaylist] -> entity
        //- [renamePlaylist] -> playlistId and playlistNewName
        //- [checkPlaylist] -> playlistId
        //- [queryPlaylist] -> playlistId
        //- [queryAllPlaylists] -> null
        //
        this.playlistId = call.argument<Number>("playlistId")
        this.entity = call.argument("entity")
        this.playlistNewName = call.argument<String>("newPlaylistName")

        //Query everything in the Background it's necessary for better performance
        viewModelScope.launch {
            //Start loading
            val dbResult: Any? = when (call.method) {
                "createPlaylist" -> createPlaylist()
                "deletePlaylist" -> deletePlaylist()
                "renamePlaylist" -> renamePlaylist()
                "checkPlaylist" -> checkPlaylist()
                "queryPlaylist" -> queryPlaylist()
                "queryAllPlaylists" -> queryAllPlaylists()
                else -> throw Exception("[$channelError]: [initPlaylistSource] have a null value.")
            }

            //Flutter UI will start, but, information still loading.
            //
            //We check [dbResult] to make sure that return will *never* be null.
            //If [dbResult] return null, something is missing and plugin will throw a error.
            if (!dbResult.isNull()) result.success(dbResult) else
                result.error(
                    channelError,
                    "[${call.method}] was called but not all parameters was defined.",
                    "Define all required parameters."
                )
        }
    }

    /*
     * This method is used to create a playlist.
     *
     * @param entity Variable that contains all information about playlist.
     * @return [Null] if some necessary variable is null or a [Long] if everything works fine.
     */
    private suspend fun createPlaylist(): Long? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        return@withContext if (entity.isNullOrEmpty()) return@withContext null
            //If none of the necessary variables are null, will return a Long(id).
            else initPlaylistDb().createPlaylist(entity!!.toPlaylistEntity())
    }

    /*
     * This method is used to delete a playlist.
     *
     * @param entity Variable that contains all information about playlist.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     */
    private suspend fun deletePlaylist(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (entity.isNullOrEmpty()) return@withContext null
        return@withContext try {
            //If none of the necessary variables are null, will return a Boolean(confirmation).
            initPlaylistDb().deletePlaylist(entity!!.toPlaylistEntity())
            true
        } catch (e: Exception) {
            //And If some of the necessary variables are null, will return a Boolean(error).
            Log.i(channelError, e.toString())
            false
        }
    }


    /*
     * This method is used to rename a playlist.
     *
     * @param playlistId Variable that define which playlist will be renamed.
     * @param playlistNewName Variable that define which name will be added to the playlist.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     */
    private suspend fun renamePlaylist(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (playlistId.isNull() && playlistNewName.isNullOrEmpty()) return@withContext null
        return@withContext try {
            //If none of the necessary variables are null, will return a Boolean(confirmation).
            initPlaylistDb().renamePlaylist(playlistId!!.toLong(), playlistNewName!!)
            true
        } catch (e: Exception) {
            //And If some of the necessary variables are null, will return a Boolean(error).
            Log.i(channelError, e.toString())
            false
        }
    }

    /*
     * This method is used to check a playlist.
     *
     * @param playlistId Variable that define which playlist will be checked.
     * @return [Null] if some necessary variable is null or a [Boolean] if everything works fine.
     */
    private suspend fun checkPlaylist(): Boolean? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (playlistId.isNull()) return@withContext null
        return@withContext try {
            //If none of the necessary variables are null, will return a Boolean(confirmation).
            initPlaylistDb().checkPlaylist(playlistId!!.toLong())
        } catch (e: Exception) {
            //And If some of the necessary variables are null, will return a Boolean(error).
            Log.i(channelError, e.toString())
            false
        }
    }

    /*
     * This method is used to query a specific playlist.
     *
     * @param playlistId Variable that define which playlist will be queried.
     * @return [Null] if some necessary variable is null or a [MutableMap<String, Any>]
     * if everything works fine.
     */
    private suspend fun queryPlaylist(): MutableMap<String, Any>? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (playlistId.isNull()) return@withContext null
        //If none of the necessary variables are null, will return a single item inside database.
        return@withContext initPlaylistDb().queryPlaylist(playlistId!!.toLong())
            ?.toPlaylistMutableMap()
    }

    /*
     * This method is used to query [All] playlists.
     *
     * @return [Null] if some necessary variable is null or a [ArrayList<MutableMap<String, Any>>]
     * if everything works fine.
     */
    private suspend fun queryAllPlaylists(): ArrayList<MutableMap<String, Any>> =
        withContext(Dispatchers.IO) {
            //Will return all items inside database.
            return@withContext initPlaylistDb().queryAllPlaylist(getDbLimit())
                .toPlaylistListAndMutableMap()
        }
}