package com.lucasjosino.on_audio_room.sources.query

import android.annotation.SuppressLint
import android.content.Context
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.lucasjosino.on_audio_room.OnAudioRoomPlugin
import com.lucasjosino.on_audio_room.databases.OnAudioDatabase
import com.lucasjosino.on_audio_room.utils.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@SuppressLint("StaticFieldLeak")
class OnQuerySource(private val context: Context) : ViewModel() {

    //Main parameters.
    private var type: Int = -1
    private var songId: Number? = null
    private var playlistId: Number? = null

    //Get database configuration.
    private fun initDb(): OnAudioDatabase = OnAudioDatabase.initDatabase(context)

    //Get database query limit.
    private fun getDbLimit(): Int = OnAudioRoomPlugin().getDbLimit(context)

    //This function is the main and initial to all calls based on [query].
    fun initQuerySource(result: MethodChannel.Result, call: MethodCall, value: Int) {
        //Method [type] *never* should be null.
        this.type = call.argument<Int>("type")
            ?: throw Exception("[$channelError]: [type] from [initQuerySource] have a null value.")

        //All others methods can be null.
        //Every [method] have your specific require variable, and cannot be null when calling the method.
        //
        //- [0] -> songId and playlistId(*)
        //- [1] -> playlistId(*)
        //
        // (*) -> Only when calling [RoomType.PLAYLIST_SONGS](type == 3))
        this.songId = call.argument<Number>("songId")
        this.playlistId = call.argument<Number>("playlistId")

        //Query everything in the Background it's necessary for better performance
        viewModelScope.launch {
            //Start loading
            val dbResult: Any? = when (value) {
                0 -> queryFrom()
                1 -> queryAllFrom()
                else -> throw Exception("[$channelError]: [initGlobalSource] have a null value.")
            }

            //Flutter UI will start, but, information still loading.
            //
            //We check [dbResult] to make sure that return will *never* be null.
            //If [dbResult] return null, something is missing and plugin will throw a error.
            if (!dbResult.isNull()) result.success(dbResult) else
                result.error(
                    channelError,
                    "[${call.method}] was called but not all parameters was defined.",
                    "Define all required parameters."
                )
        }
    }

    /*
     * This method is used to query a specific song(entity) from a specific database.
     *
     * @param songId Variable that define which song will be queried.
     * @param playlistId Variable that define which playlist the song will be queried.
     * @return [Null] if some necessary variable is null or a [MutableMap<String, Any>]
     * if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun queryFrom(): MutableMap<String, Any>? = withContext(Dispatchers.IO) {
        //Check to make sure that none of the necessary variables are null.
        if (songId.isNull() || type == 3 && playlistId.isNull()) return@withContext null

        //Now we use [type] to define the database.
        //
        //- [0] -> Favorites(queryFromFavorites)
        //- [1] -> LastPlayed(queryFromLastPlayed)
        //- [2] -> MostPlayed(queryFromMostPlayed)
        //- [3] -> Playlist songs(queryFromPlaylist)
        //
        return@withContext when (type) {
            0 -> initDb().onFavoritesDao().queryFromFavorites(songId!!.toLong())?.toFavMutableMap()
            1 -> initDb().onLastPlayedDao().queryFromLastPlayed(songId!!.toLong())
                ?.toLastPlayedMutableMap()
            2 -> initDb().onMostPlayedDao().queryFromMostPlayed(songId!!.toLong())
                ?.toMostPlayedMutableMap()
            3 -> initDb().onPlaylistSongDao()
                .queryFromPlaylist(playlistId!!.toLong(), songId!!.toLong())
                ?.toPlaylistSongsMutableMap()
            //[queryFrom] is null when [type] is null. Technically can't be null. But, if is will
            //throw this error:
            else -> throw NullPointerException("[$channelError]: [queryFrom] have a null value.")
        }
    }

    /*
     * This method is used to query [All] songs(entities) from a specific database.
     *
     * @param playlistId Variable that define which playlist the songs will be queried.
     * @return [Null] if some necessary variable is null or a [MutableMap<String, Any>]
     * if everything works fine.
     * @throws NullPointerException
     */
    private suspend fun queryAllFrom(): ArrayList<MutableMap<String, Any>>? =
        withContext(Dispatchers.IO) {
            //Check to make sure that none of the necessary variables are null.
            if (type == 3 && playlistId.isNull()) return@withContext null

            //Now we use [type] to define the database.
            //
            //- [0] -> Favorites(queryAllFromFavorites)
            //- [1] -> LastPlayed(queryAllFromLastPlayed)
            //- [2] -> MostPlayed(queryAllFromMostPlayed)
            //- [3] -> Playlist songs(queryAllFromPlaylist)
            //
            return@withContext when (type) {
                0 -> initDb().onFavoritesDao().queryAllFromFavorites(getDbLimit())
                    .toFavListAndMutableMap()
                1 -> initDb().onLastPlayedDao().queryAllFromLastPlayed(getDbLimit())
                    .toLPListAndMutableMap()
                2 -> initDb().onMostPlayedDao().queryAllFromMostPlayed(getDbLimit())
                    .toMPListAndMutableMap()
                3 -> initDb().onPlaylistSongDao()
                    .queryAllFromPlaylist(playlistId!!.toLong(), getDbLimit())
                    .toPlaylistSongsListAndMutableMap()
                //[queryFrom] is null when [type] is null. Technically can't be null. But, if is will
                //throw this error:
                else -> throw NullPointerException("[$channelError]: [queryAllFrom] have a null value.")
            }
        }
}