/*
=============
Author: Lucas Josino
Github: https://github.com/LucasPJS
Website: https://lucasjosino.com/
=============
Plugin/Id: on_audio_room#4
Homepage: https://github.com/LucasPJS/on_audio_room
Pub: https://pub.dev/packages/on_audio_room
License: https://github.com/LucasPJS/on_audio_room/blob/main/LICENSE
Copyright: © 2021, Lucas Josino. All rights reserved.
=============
*/

part of on_audio_room;

/// Interface and Main method for use [on_audio_room].
///
/// [on_audio_room] is a [Flutter](https://flutter.dev/) Plugin used to create a
/// database for storage audio sections.
///
/// RoomTypes:
///
/// * Favorites.
/// * Last Played.
/// * MostPlayed.
/// * Playlists.
///     * Playlist songs.
///
/// See more:
///
/// Homepage: [GitHub](https://github.com/LucasPJS/on_audio_room) <br>
/// Documentation: [API](https://pub.dev/documentation/on_audio_room/latest/) <br>
///
/// Any problem? [Issues](https://github.com/LucasPJS/on_audio_room/issues) <br>
/// Any suggestion? [Pull request](https://github.com/LucasPJS/on_audio_room/pulls)
class OnAudioRoom {
  //Dart <-> Kotlin communication
  static const String _CHANNEL_ID = "com.lucasjosino.on_audio_room";
  static const MethodChannel _channel = const MethodChannel(_CHANNEL_ID);

  //Database methods

  /// Used to define the database name.
  ///
  /// ### Before usage:
  ///
  /// **If you set/change database name after load/call any [Add/Delete/Query]**
  /// **method will lose all information and the old database will still existing**
  /// **and consuming memory.**
  ///
  /// Parameters:
  ///
  /// * [databaseName] this will be the name of your database.
  ///
  /// Important:
  ///
  /// * If you don't define a name to the database, the plugin will be set the
  /// default name [on_audio_db].
  /// * It's recommended set your [own] database name, this plugin is public
  /// and any one can use it. Avoid future problems.
  /// **(Every app has your own database file, but, you never know what can happen.).**
  ///
  /// Return:
  ///
  /// * If the name has added/redefined will return [true], if some problem occurred
  /// will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when [databaseName] is null or other name
  /// was defined before.
  ///
  /// See too:
  ///
  /// * [getDatabaseName]
  /// * [resetDatabaseName]
  Future<bool> setDatabaseName(String databaseName) async =>
      await _channel.invokeMethod("setDatabaseName", {
        "databaseName": databaseName,
      });

  /// Used to define the database limit.
  ///
  /// Parameters:
  ///
  /// * [databaseLimit] this will be the limit of your database query.
  ///
  /// Important:
  ///
  /// * If you don't define a limit to the database, the plugin will be set the
  /// default limit [50].
  ///
  /// Return:
  ///
  /// * If the limit has added/redefined will return [true], if some problem occurred
  /// will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when [databaseLimit] is null or other limit
  /// was defined before.
  ///
  /// See too:
  ///
  /// * [getDatabaseLimit]
  /// * [resetDatabaseLimit]
  Future<bool> setDatabaseLimit(int databaseLimit) async =>
      await _channel.invokeMethod("setDatabaseLimit", {
        "databaseLimit": databaseLimit,
      });

  /// Used to get the database name.
  ///
  /// Important:
  ///
  /// * If you never defined a name to the database, the plugin will be return the
  /// default name [on_audio_db].
  ///
  /// See too:
  ///
  /// * [setDatabaseName]
  /// * [resetDatabaseName]
  Future<String> get getDatabaseName async =>
      await _channel.invokeMethod("getDatabaseName");

  /// Used to get the database limit.
  ///
  /// Important:
  ///
  /// * If you never defined a limit to the database, the plugin will be return the
  /// default limit [50].
  ///
  /// See too:
  ///
  /// * [setDatabaseLimit]
  /// * [resetDatabaseLimit]
  Future<int> get getDatabaseLimit async =>
      await _channel.invokeMethod("getDatabaseLimit");

  /// Used to get the database status.
  ///
  /// Return:
  ///
  /// * Will return [true] if the database is open and [false] if is close.
  Future<bool> get getDatabaseStatus async =>
      await _channel.invokeMethod("getDatabaseStatus");

  /// Used to reset the database name.
  ///
  /// ### Before usage:
  ///
  /// **If you reset the database name after load/call any [Add/Delete/Query]**
  /// **method will lose all information and the old database will still existing**
  /// **and consuming memory.**
  ///
  /// Return:
  ///
  /// * Will return [true] if the old database name has been set to default,
  /// if some problem occurred will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when was never been defined before.
  ///
  /// See too:
  ///
  /// * [setDatabaseName]
  /// * [getDatabaseName]
  Future<bool> get resetDatabaseName async =>
      await _channel.invokeMethod("resetDatabaseName");

  /// Used to reset the database limit.
  ///
  /// Return:
  ///
  /// * Will return [true] if the old database limit has been set to default,
  /// if some problem occurred will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when was never been defined before.
  ///
  /// See too:
  ///
  /// * [setDatabaseName]
  /// * [getDatabaseName]
  Future<bool> get resetDatabaseLimit async =>
      await _channel.invokeMethod("resetDatabaseLimit");

  /// Used to reset [ALL] the database.
  ///
  /// ### Before usage:
  ///
  /// **After deleting the rows, Room will set a WAL checkpoint and run VACUUM.**
  /// **This means that the data is completely erased. The space will be reclaimed**
  /// **by the system if the amount surpasses the threshold of database file size.**
  /// [Room Documentation](https://bit.ly/3vGyqdV).
  ///
  /// Return:
  ///
  /// * Will return [true] if the database has been cleaned.
  Future<bool> get resetAllDatabase async =>
      await _channel.invokeMethod("resetAllDatabase");

  /// Used to close the database.
  ///
  /// Return:
  ///
  /// * Will return [true] if the database has been close.
  Future<bool> get closeDatabase async =>
      await _channel.invokeMethod("closeDatabase");

  //
  // Future<bool> get deleteDatabase async =>
  //     await _channel.invokeMethod("deleteDatabase");

  //Global methods

  /// Used to add a specific song(entity) to a specific database.
  ///
  /// Parameters:
  ///
  /// * [addTo] this will define which database the entity will be added.
  /// * [entity] this is the entity with all information about song.
  /// * [playlistId] this will be the id of the playlist [*].
  ///
  /// [*] Playlist it's only required when using [RoomType.PLAYLIST_SONG].
  ///
  /// Important:
  ///
  /// * For now if you add two entities with the same information, will be added.
  /// You can use [checkIn] to make sure that the song was already added.
  ///
  /// Usage:
  ///
  /// * A [entity] is a [Map<dynamic, dynamic>]. You have two options:
  ///
  ///     1. Create a manual entity.
  ///
  ///     2. Use [on_audio_query] extension to create the entity.
  ///
  /// Examples:
  ///
  /// 1- Manual
  ///
  /// ```dart
  /// Map<dynamic, dynamic> entityMap = {
  ///   "last_data": song.data,
  ///   "display_name": song.displayName,
  ///   "id": song.id,
  ///   "album": song.album,
  ///   "album_id": song.albumId,
  ///   "artist": song.artist,
  ///   "artist_id": song.artistId,
  ///   "date_added": song.dateAdded,
  ///   "duration": song.duration,
  ///   "title": song.title,
  ///   "artwork": song.artwork,
  /// };
  /// ```
  ///
  /// 2- [OnAudioQuery](https://github.com/LucasPJS/on_audio_query)
  ///
  /// ```dart
  /// List<SongModel> listOfSongs;
  /// //toFavoritesEntity()
  /// //toLastPlayedEntity()
  /// //toMostPlayedEntity()
  /// //toPlaylistSongsEntity()
  /// Map<dynamic, dynamic> entityMap = listOfSongs[index].toFavoritesEntity();
  /// ```
  ///
  /// Return:
  ///
  /// * If the song(entity) was added, will return [int] that represent the song id
  /// inside the database, if some problem occurred, will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when [entity] is null or when [RoomType.PLAYLIST_SONG]
  /// is called and [playlistId] is null.
  ///
  /// See too:
  ///
  /// * [addAllTo]
  /// * [checkIn]
  Future<int> addTo(RoomType addTo, Map<dynamic, dynamic> entity,
      {int? playlistId}) async {
    if (addTo == RoomType.PLAYLIST_SONG && playlistId == null)
      throw Exception("Cannot add [entity] to a undefined [playlist]\n"
          "Define the [playlistId] parameter.");
    final int addResult = await _channel.invokeMethod("addTo", {
      "type": addTo.index,
      "entity": entity,
      "playlistId": playlistId,
    });
    return addResult;
  }

  /// Used to add a list of song(entity) to a specific database.
  ///
  /// Parameters:
  ///
  /// * [addAllTo] this will define which database the list of entity will be added.
  /// * [entity] this is the list of entity with all information about songs.
  /// * [playlistId] this will be the id of the playlist [*].
  ///
  /// [*] Playlist it's only required when using [RoomType.PLAYLIST_SONG].
  ///
  /// Important:
  ///
  /// * For now if you add two entities with the same information, will be added.
  /// You can use [checkIn] to make sure that the song was already added.
  ///
  /// Usage:
  ///
  /// * A [entity] is a [Map<dynamic, dynamic>]. You have two options:
  ///
  ///     1. Create a manual entity.
  ///
  ///     2. Use [on_audio_query] extension to create the entity.
  ///
  /// Examples:
  ///
  /// 1- Manual
  ///
  /// ```dart
  /// Map<dynamic, dynamic> entityMap = {
  ///   "last_data": song.data,
  ///   "display_name": song.displayName,
  ///   "id": song.id,
  ///   "album": song.album,
  ///   "album_id": song.albumId,
  ///   "artist": song.artist,
  ///   "artist_id": song.artistId,
  ///   "date_added": song.dateAdded,
  ///   "duration": song.duration,
  ///   "title": song.title,
  ///   "artwork": song.artwork,
  /// };
  /// ```
  ///
  /// 2- [OnAudioQuery](https://github.com/LucasPJS/on_audio_query)
  ///
  /// ```dart
  /// List<SongModel> listOfSongs;
  /// //toFavoritesEntity()
  /// //toLastPlayedEntity()
  /// //toMostPlayedEntity()
  /// //toPlaylistSongsEntity()
  /// Map<dynamic, dynamic> entityMap = listOfSongs[index].toFavoritesEntity();
  /// ```
  ///
  /// Return:
  ///
  /// * If the list of song(entity) was added, will return a list of [int] that
  /// represent the songs ids inside the database, if some problem occurred,
  /// will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when some of the [entity] is null or when
  /// [RoomType.PLAYLIST_SONG] is called and [playlistId] is null.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [checkIn]
  Future<List<int>> addAllTo(
      RoomType addAllTo, List<Map<dynamic, dynamic>> listOfEntity,
      {int? playlistId}) async {
    if (addAllTo == RoomType.PLAYLIST_SONG && playlistId == null)
      throw Exception("Cannot add [listOfEntity] to a undefined [playlist].\n"
          "Define the [playlistId] parameter.");
    final List<dynamic> addAllResult = await _channel.invokeMethod("addAllTo", {
      "type": addAllTo.index,
      "listOfEntity": listOfEntity,
      "playlistId": playlistId,
    });
    return addAllResult.cast<int>();
  }

  /// Used to delete a specific song(entity) from a specific database.
  ///
  /// Parameters:
  ///
  /// * [deleteFrom] this will define which database the entity will be deleted.
  /// * [songId] this is the id of the song that will be deleted.
  /// * [playlistId] this will be the id of the playlist [*].
  ///
  /// [*] Playlist it's only required when using [RoomType.PLAYLIST_SONG].
  ///
  /// Return:
  ///
  /// * If the song was deleted, will return [true], if not, will return [false].
  ///
  /// See too:
  ///
  /// * [deleteAllFrom]
  /// * [checkIn]
  Future<bool> deleteFrom(RoomType deleteFrom, int songId,
      {int? playlistId}) async {
    if (deleteFrom == RoomType.PLAYLIST_SONG && playlistId == null)
      throw Exception("Cannot delete [song] from a undefined [playlist].\n"
          "Define the [playlistId] parameter.");
    final bool deleteResult = await _channel.invokeMethod("deleteFrom", {
      "type": deleteFrom.index,
      "songId": songId,
      "playlistId": playlistId,
    });
    return deleteResult;
  }

  /// Used to delete all the entities from a specific database.
  ///
  /// Parameters:
  ///
  /// * [deleteAllFrom] this will define which database will be cleaned.
  ///
  /// Return:
  ///
  /// * If the database was cleaned, will return [true], if not, will return [false].
  ///
  /// See too:
  ///
  /// * [deleteFrom]
  /// * [checkIn]
  Future<bool> deleteAllFrom(RoomType deleteAllFrom) async {
    final bool deleteAllResult = await _channel.invokeMethod("deleteAllFrom", {
      "type": deleteAllFrom.index,
    });
    return deleteAllResult;
  }

  /// Used to check if a song(entity) was added to a specific database.
  ///
  /// Parameters:
  ///
  /// * [checkIn] this will define which database will be checked.
  /// * [songId] this is the id of the song that will be checked.
  /// * [playlistId] this will be the id of the playlist [*].
  ///
  /// [*] Playlist it's only required when using [RoomType.PLAYLIST_SONG].
  ///
  /// Return:
  ///
  /// * If the song(entity) exist in the database, will return [true], if not,
  /// will return [false] and if some problem occurred, will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when some of the [songId] is null or when
  /// [RoomType.PLAYLIST_SONG] is called and [playlistId] is null.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  Future<bool> checkIn(RoomType checkIn, int songId, {int? playlistId}) async {
    if (checkIn == RoomType.PLAYLIST_SONG && playlistId == null)
      throw Exception("Cannot check [song] from a undefined [playlist].\n"
          "Define the [playlistId] parameter.");
    final bool checkResult = await _channel.invokeMethod("checkIn", {
      "type": checkIn.index,
      "songId": songId,
      "playlistId": playlistId,
    });
    return checkResult;
  }

  /// Used to query all ids from a specific database.
  ///
  /// Parameters:
  ///
  /// * [queryFrom] this will define which database will be queried.
  /// * [playlistId] this will be the id of the playlist [*].
  ///
  /// [*] Playlist it's only required when using [RoomType.PLAYLIST_SONG].
  ///
  /// Return:
  ///
  /// * Will return a list of [int] that represent all ids from database. If
  /// some problem occurred, will throw a error.
  ///
  /// Throw:
  ///
  /// * Will throw a error when [RoomType.PLAYLIST_SONG] is called and
  /// [playlistId] is null.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  Future<List<int>> queryAllIdsFrom(RoomType queryFrom,
      {int? playlistId}) async {
    if (queryFrom == RoomType.PLAYLIST_SONG && playlistId == null)
      throw Exception("Cannot query [entities] from a undefined [playlist].\n"
          "Define the [playlistId] parameter.");
    final List<dynamic> queryAllIdsResult =
        await _channel.invokeMethod("queryAllIdsFrom", {
      "type": queryFrom.index,
      "playlistId": playlistId,
    });
    return queryAllIdsResult.cast<int>();
  }

  //Query methods

  //Query favorites methods

  /// Used to query a specific song(entity) from [favorites] database.
  ///
  /// Use [checkIn] to know if song exist on [favorites] database.
  ///
  /// Parameters:
  ///
  /// * [songId] this is the id of the song that will be queried.
  ///
  /// Return:
  ///
  /// * Will return a [FavoritesEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<FavoritesEntity> queryFromFavorites(int songId) async {
    final Map<dynamic, dynamic> queryFavoritesResult =
        await _channel.invokeMethod("queryFromFavorites", {
      "type": 0,
      "songId": songId,
    });
    return FavoritesEntity(queryFavoritesResult);
  }

  /// Used to query all songs(entities) from [favorites] database.
  ///
  /// Use [checkIn] to know if song exist on [favorites] database.
  ///
  /// Parameters:
  ///
  /// * [queryLimit] this will limit the amount of entities queried. Taking less
  /// time to run and giving a better performance.
  ///
  /// Return:
  ///
  /// * Will return a list of [FavoritesEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<List<FavoritesEntity>> queryAllFromFavorites({int? queryLimit}) async {
    final List<dynamic> queryAllFavoritesResult =
        await _channel.invokeMethod("queryAllFromFavorites", {
      "type": 0,
      "queryLimit": queryLimit,
    });
    return queryAllFavoritesResult
        .map((favoritesInfo) => FavoritesEntity(favoritesInfo))
        .toList();
  }

  //Query lastPlayed methods

  /// Used to query a specific song(entity) from [last_played] database.
  ///
  /// Use [checkIn] to know if song exist on [last_played] database.
  ///
  /// Parameters:
  ///
  /// * [songId] this is the id of the song that will be queried.
  ///
  /// Return:
  ///
  /// * Will return a [LastPlayedEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<LastPlayedEntity> queryFromLastPlayed(int songId) async {
    final Map<dynamic, dynamic> queryLastPlayedResult =
        await _channel.invokeMethod("queryFromLastPlayed", {
      "type": 1,
      "songId": songId,
    });
    return LastPlayedEntity(queryLastPlayedResult);
  }

  /// Used to query all songs(entities) from [last_played] database.
  ///
  /// Use [checkIn] to know if song exist on [last_played] database.
  ///
  /// Parameters:
  ///
  /// * [queryLimit] this will limit the amount of entities queried. Taking less
  /// time to run and giving a better performance.
  ///
  /// Return:
  ///
  /// * Will return a list of [LastPlayedEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<List<LastPlayedEntity>> queryAllFromLastPlayed(
      {int? queryLimit}) async {
    final List<dynamic> queryAllLastPlayedResult =
        await _channel.invokeMethod("queryAllFromLastPlayed", {
      "type": 1,
      "queryLimit": queryLimit,
    });
    return queryAllLastPlayedResult
        .map((lastPlayedInfo) => LastPlayedEntity(lastPlayedInfo))
        .toList();
  }

  //Query mostPlayed methods

  /// Used to query a specific song(entity) from [most_played] database.
  ///
  /// Use [checkIn] to know if song exist on [most_played] database.
  ///
  /// Parameters:
  ///
  /// * [songId] this is the id of the song that will be queried.
  ///
  /// Return:
  ///
  /// * Will return a [MostPlayedEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<MostPlayedEntity> queryFromMostPlayed(int songId) async {
    final Map<dynamic, dynamic> queryMostPlayedResult =
        await _channel.invokeMethod("queryFromMostPlayed", {
      "type": 2,
      "songId": songId,
    });
    return MostPlayedEntity(queryMostPlayedResult);
  }

  /// Used to query all songs(entities) from [most_played] database.
  ///
  /// Use [checkIn] to know if song exist on [most_played] database.
  ///
  /// Parameters:
  ///
  /// * [queryLimit] this will limit the amount of entities queried. Taking less
  /// time to run and giving a better performance.
  ///
  /// Return:
  ///
  /// * Will return a list of [MostPlayedEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<List<MostPlayedEntity>> queryAllFromMostPlayed(
      {int? queryLimit}) async {
    final List<dynamic> queryAllMostPlayedResult =
        await _channel.invokeMethod("queryAllFromMostPlayed", {
      "type": 2,
      "queryLimit": queryLimit,
    });
    return queryAllMostPlayedResult
        .map((mostPlayedInfo) => MostPlayedEntity(mostPlayedInfo))
        .toList();
  }

  //Query playlist_songs methods

  /// Used to query a specific song(entity) from [playlist_songs] database.
  ///
  /// Use [checkIn] to know if song exist on [playlist_songs] database.
  ///
  /// Parameters:
  ///
  /// * [songId] this is the id of the song that will be queried.
  /// * [playlistId] this will be the id of the playlist.
  ///
  /// Return:
  ///
  /// * Will return a [PlaylistSongsEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<PlaylistSongsEntity> queryFromPlaylist(
      int playlistId, int songId) async {
    final Map<dynamic, dynamic> queryInPlaylistResult =
        await _channel.invokeMethod("queryInPlaylist", {
      "type": 3,
      "playlistId": playlistId,
      "songId": songId,
    });
    return PlaylistSongsEntity(queryInPlaylistResult);
  }

  /// Used to query all songs(entities) from [playlist_songs] database.
  ///
  /// Use [checkIn] to know if song exist on [playlist_songs] database.
  ///
  /// Parameters:
  ///
  /// * [playlistId] this is the id of the playlist that will be queried.
  /// * [queryLimit] this will limit the amount of entities queried. Taking less
  /// time to run and giving a better performance.
  ///
  /// Return:
  ///
  /// * Will return a list of [PlaylistSongsEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<List<PlaylistSongsEntity>> queryAllFromPlaylist(int playlistId,
      {int? queryLimit}) async {
    final List<dynamic> queryAllInPlaylistResult =
        await _channel.invokeMethod("queryAllFromPlaylist", {
      "type": 3,
      "playlistId": playlistId,
      "queryLimit": queryLimit,
    });
    return queryAllInPlaylistResult
        .map((playlistSongsInfo) => PlaylistSongsEntity(playlistSongsInfo))
        .toList();
  }

  //Playlist methods

  /// Used to create a internal playlist.
  ///
  /// Parameters:
  ///
  /// * [entity] this is the entity with all information about playlist.
  ///
  /// Usage:
  ///
  /// * To create a internal playlist you will only need three information.
  ///
  ///     * Playlist Name
  ///
  ///     * Playlist Date Added
  ///
  ///     * Playlist Modified
  ///
  /// * You can use [toPlaylistEntity] method extension from a [String].
  ///
  /// Example:
  ///
  /// ```dart
  /// Map<dynamic, dynamic> playlistEntity = "My playlist 0 0".toPlaylistEntity();
  /// //This is the same as:
  /// Map<dynamic, dynamic> playlistEntity = {
  ///   "playlist_id": "My playlist",
  ///   "playlist_date_added": "0",
  ///   "playlist_date_modified": "0",
  /// };
  /// ```
  ///
  /// Return:
  ///
  /// * Will return a [int] representing the playlist Id.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<int> createPlaylist(Map<dynamic, dynamic> entity) async {
    final int createResult = await _channel.invokeMethod("createPlaylist", {
      "entity": entity,
    });
    return createResult;
  }

  /// Used to delete a internal playlist.
  ///
  /// Parameters:
  ///
  /// * [entity] this is the entity with all information about playlist.
  ///
  /// Usage:
  ///
  /// * To create a internal playlist you will only need three information.
  ///
  ///     * Playlist Name
  ///
  ///     * Playlist Date Added
  ///
  ///     * Playlist Modified
  ///
  /// * You can use [toPlaylistEntity] method extension from a [String].
  ///
  /// Example:
  ///
  /// ```dart
  /// Map<dynamic, dynamic> playlistEntity = "My playlist 0 0".toPlaylistEntity();
  /// //This is the same as:
  /// Map<dynamic, dynamic> playlistEntity = {
  ///   "playlist_id": "My playlist",
  ///   "playlist_date_added": "0",
  ///   "playlist_date_modified": "0",
  /// };
  /// ```
  ///
  /// Return:
  ///
  /// * Will return [true] if the playlist has delete and [false] if not.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<bool> deletePlaylist(Map<dynamic, dynamic> entity) async {
    final bool deleteResult = await _channel.invokeMethod("deletePlaylist", {
      "entity": entity,
    });
    return deleteResult;
  }

  /// Used to rename a internal playlist.
  ///
  /// Parameters:
  ///
  /// * [playlistId] this is the id of the playlist that will be renamed.
  /// * [newPlaylistName] this is the new name of the playlist.
  ///
  /// Return:
  ///
  /// * Will return [true] if the playlist has renamed and [false] if not.
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<bool> renamePlaylist(int playlistId, String newPlaylistName) async {
    final bool renameResult = await _channel.invokeMethod("renamePlaylist", {
      "playlistId": playlistId,
      "newPlaylistName": newPlaylistName,
    });
    return renameResult;
  }

  /// Used to query a specific playlist(entity) from [playlists] database.
  ///
  /// Parameters:
  ///
  /// * [playlistId] this is the id of the playlist that will be queried.
  ///
  /// Return:
  ///
  /// * Will return a [PlaylistEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<PlaylistEntity> queryPlaylist(int playlistId) async {
    final Map<dynamic, dynamic> queryPlaylistResult =
        await _channel.invokeMethod("queryPlaylist", {
      "playlistId": playlistId,
    });
    return PlaylistEntity(queryPlaylistResult);
  }

  /// Used to query all playlist(entities) from [playlists] database.
  ///
  /// Parameters:
  ///
  /// * [queryLimit] this will limit the amount of entities queried. Taking less
  /// time to run and giving a better performance.
  ///
  /// Return:
  ///
  /// * Will return a list of [PlaylistEntity].
  ///
  /// See too:
  ///
  /// * [addTo]
  /// * [addAllTo]
  /// * [deleteFrom]
  /// * [deleteAllFrom]
  /// * [queryAllIdsFrom]
  Future<List<PlaylistEntity>> queryAllPlaylists({int? queryLimit}) async {
    final List<dynamic> queryAllPlaylistsResult =
        await _channel.invokeMethod("queryAllPlaylists", {
      "queryLimit": queryLimit,
    });
    return queryAllPlaylistsResult
        .map((playlistInfo) => PlaylistEntity(playlistInfo))
        .toList();
  }
}
