///This method will convert a [int] number into a [String] using date format.
extension OnTimeFormatter on int {
  ///This method will convert a [int] number into a [String] using date format.
  ///
  /// Types:
  ///
  /// * Days
  /// * Hours
  /// * Minutes
  /// * Seconds
  /// * Milliseconds
  ///
  /// Important:
  ///
  /// * If none of the parameters is [true] the format will be Minute:Seconds.
  ///
  /// Examples:
  ///
  /// 1-
  ///
  /// ```dart
  /// String convert = 100000000.toTimeFormat(showDays: true, showMilli: true);
  /// print(convert); //Format: 02:03:46:40:40
  /// ```
  ///
  /// 2-
  ///
  /// ```dart
  /// String convert = 100000000.toTimeFormat(showDays: false, showMilli: false);
  /// print(convert); //Format: 46:40
  /// ```
  ///
  /// 3-
  ///
  /// ```dart
  /// String convert = 100000000.toTimeFormat(showHours: true);
  /// print(convert); //Format: 03:46:40
  /// ```
  String toTimeFormat({
    bool? showDays,
    bool? showHours,
    bool? showMilli,
  }) {
    Duration time = Duration(milliseconds: this);
    String twoDigits(int n) => n.toString().padLeft(2, "0");
    String twoDigitHours = twoDigits(time.inHours.remainder(24));
    String twoDigitMinutes = twoDigits(time.inMinutes.remainder(60));
    String twoDigitSeconds = twoDigits(time.inSeconds.remainder(60));
    String twoDigitMilli = twoDigits(time.inMilliseconds.remainder(60));
    String defaultValue = "$twoDigitMinutes:$twoDigitSeconds";

    if (showMilli != null && showMilli)
      defaultValue = defaultValue + ":$twoDigitMilli";
    if (showHours != null && showHours)
      defaultValue = "$twoDigitHours:" + defaultValue;
    if (showDays != null && showDays) {
      showHours = true;
      defaultValue = "${time.inDays}:$twoDigitHours:" + defaultValue;
    }
    return defaultValue;
  }
}

///This method will convert a [Map<dynamic, dynamic>] into a [PlaylistEntity].
extension OnEntityFormatter on String {
  /// This method will convert a [Map<dynamic, dynamic>] into a [PlaylistEntity].
  ///
  /// Important:
  ///
  /// * You have to follow always the same sequence:
  ///
  ///     * Playlist Name
  ///
  ///     * Playlist Date Added
  ///
  ///     * Playlist Date Modified
  ///
  /// Example:
  ///
  /// ```dart
  /// Map<dynamic, dynamic> playlistEntity = "My playlist 100000000 100000000".toPlaylistEntity();
  /// //This is the same as:
  /// Map<dynamic, dynamic> playlistEntity = {
  ///   "playlist_id": "My playlist",
  ///   "playlist_date_added": "100000000",
  ///   "playlist_date_modified": "100000000",
  /// };
  /// ```
  Map<dynamic, dynamic> toPlaylistEntity() {
    assert(this.isNotEmpty, "Cannot convert empty string to [PlaylistEntity]");
    List<String> splitString = this.split(" ");
    assert(!(splitString.length > 3), "String length is bigger than 3");
    assert(!(splitString.length < 3), "String length is less than 3");
    Map<dynamic, dynamic> tempMap = {
      "playlist_name": splitString[0],
      "playlist_date_added": splitString[1],
      "playlist_date_modified": splitString[2],
    };
    return tempMap;
  }
}
